<?php
if ( class_exists( 'MailPress' ) && !class_exists( 'MailPress_delete_inactive_users' ) )
{
/*
Plugin Name: MailPress_delete_inactive_users
Plugin URI: http://blog.mailpress.org/tutorials/add-ons/delete_inactive_users/
Description: Users : delete inactive/bounced/unsubscribed mp users
Version: 7.1
*/

class MailPress_delete_inactive_users
{
	const option_name = 'MailPress_delete_inactive_users';

	const process_name		= 'mp_process_delete_inactive_users';

	const bt = 132;

	function __construct()
	{
		add_action( self::process_name, 				array( $this, 'process' ) );

		$config = get_option( self::option_name );
		if ( 'wpcron' == $config['batch_mode'] )
		{	
			add_action( 'MailPress_schedule_delete_inactive_users', 	array( __CLASS__, 'schedule' ) );
		}

		if ( is_admin() )
		{
		// for install
			register_activation_hook( plugin_basename( __FILE__ ), 	array( __CLASS__, 'install' ) );
			register_deactivation_hook( plugin_basename( __FILE__ ),	array( __CLASS__, 'uninstall' ) );
		// for link on plugin page
			add_filter( 'plugin_action_links', 				array( __CLASS__, 'plugin_action_links' ), 10, 2 );

		// for settings
			add_filter( 'MailPress_scripts', 				array( __CLASS__, 'scripts' ), 8, 2 );
			add_filter( 'MailPress_settings_tab', 			array( __CLASS__, 'settings_tab' ), 20, 1 );
		// for settings batches
			add_filter( 'MailPress_settings_batches_help',		array( __CLASS__, 'settings_batches_help' ), 20, 1 );
			add_action( 'MailPress_settings_batches_update',	array( __CLASS__, 'settings_batches_update' ), 20 );
			add_action( 'MailPress_settings_batches_form',		array( __CLASS__, 'settings_batches_form' ), 20 );
		}	
	}

// process
	public static function process()
	{
		self::schedule();

		global $wpdb;

		$config = get_option( self::option_name );
		if ( !$config ) return false;

		MP_::no_abort_limit();

		$date = date( 'Y-m-d', current_time( 'timestamp' ) - ( $config['days'] * 86400 ) );

		$ids = $wpdb->get_results( $wpdb->prepare( "SELECT id FROM $wpdb->mp_users WHERE status in ( 'waiting', 'inactive', 'unsubscribed' ) AND DATE( laststatus ) < %s;", $date ) );

		foreach( $ids as $id ) MP_User::delete( $id->id );
	}

// schedule
	public static function schedule()
	{
		$config = get_option( self::option_name );
		$now4cron = current_time( 'timestamp', 'gmt' );

		if ( !wp_next_scheduled( self::process_name ) ) 
			wp_schedule_single_event( $now4cron + $config['every'] * 86400, self::process_name );
	}

////  ADMIN  ////
////  ADMIN  ////
////  ADMIN  ////
////  ADMIN  ////

// for install
	public static function install() 
	{
		self::uninstall();

		do_action( 'MailPress_schedule_delete_inactive_users' );
	}

	public static function uninstall() 
	{
		wp_clear_scheduled_hook( self::process_name );
	}

// for link on plugin page
	public static function plugin_action_links( $links, $file )
	{
		return MailPress::plugin_links( $links, $file, plugin_basename( __FILE__ ), 'batches' );
	}

// for settings
	public static function scripts( $scripts, $screen ) 
	{
		if ( $screen != MailPress_page_settings ) return $scripts;

		wp_register_script( 'mp-delete-inactive-users', 	'/' . MP_PATH . 'mp-admin/js/settings_delete_inactive_users.js', array(), false, 1 );
		$scripts[] = 'mp-delete-inactive-users';
		return $scripts;
	}

	public static function settings_tab( $tabs )
	{
		$tabs['batches'] = __( 'Batches', 'MailPress' );
		return $tabs;
	}

// for settings batches
	public static function settings_batches_help( $content )
	{
		include ( MP_ABSPATH . 'mp-admin/includes/settings/batches/delete_inactive_users/help.php' );
		return $content;
	}

	public static function settings_batches_update()
	{
		include ( MP_ABSPATH . 'mp-admin/includes/settings/batches/delete_inactive_users/update.php' );
	}

	public static function settings_batches_form()
	{
		include ( MP_ABSPATH . 'mp-admin/includes/settings/batches/delete_inactive_users/form.php' );
	}

}
new MailPress_delete_inactive_users();
}